package io.sixminutes.ridicule.monitor

import android.content.ContentResolver
import android.content.pm.PackageManager
import de.robv.android.xposed.IXposedHookLoadPackage
import de.robv.android.xposed.XC_MethodHook
import de.robv.android.xposed.XposedBridge
import de.robv.android.xposed.XposedHelpers
import de.robv.android.xposed.callbacks.XC_LoadPackage
import java.io.File

class DeviceIdMonitor : IXposedHookLoadPackage {

    companion object {
        private const val TARGET_PACKAGE = "io.sixminutes.breakingnews"
        private const val LOG_TAG = "DeviceIdTracker"
        private const val STACK_DEPTH = 10 // 堆栈跟踪深度
    }

    override fun handleLoadPackage(lpparam: XC_LoadPackage.LoadPackageParam) {
        XposedBridge.log("[DeviceIdTracker] Module loaded for ${lpparam.packageName}")
//        if (lpparam.packageName != TARGET_PACKAGE) return

        hookTelephonyApis(lpparam.classLoader)
        hookSystemApis(lpparam.classLoader)
        hookReflectiveCalls(lpparam.classLoader)
        bypassPermissionChecks(lpparam.classLoader)

        // Hook ClassLoader.loadClass 检测动态类加载
        XposedHelpers.findAndHookMethod(
            "java.lang.ClassLoader",
            lpparam.classLoader,
            "loadClass",
            String::class.java,
            Boolean::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    val className = param.args[0] as String
                    if (className.contains("TelephonyManager") || className.contains("Build")) {
                        logCall("ClassLoader.loadClass", "className=$className", param.result)
                    }
                }
            }
        )

        // 补充 Hook MAC 地址（WiFi/BT）
        XposedHelpers.findAndHookMethod(
            "android.net.wifi.WifiInfo",
            lpparam.classLoader,
            "getMacAddress",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall("WifiInfo", "getMacAddress", param.result)
                }
            }
        )

        XposedHelpers.findAndHookMethod(
            "android.bluetooth.BluetoothAdapter",
            lpparam.classLoader,
            "getAddress",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall("BluetoothAdapter", "getAddress", param.result)
                }
            }
        )

//        // Hook 通过文件读取设备信息（如 /proc/cpuinfo）
//        XposedHelpers.findAndHookMethod(
//            "java.io.File",
//            lpparam.classLoader,
//            "readLines",
//            object : XC_MethodHook() {
//                override fun afterHookedMethod(param: MethodHookParam) {
//                    val path = (param.thisObject as File).absolutePath
//                    if (path.contains("proc/cpuinfo") || path.contains("serial")) {
//                        logCall("File.readLines", "path=$path", param.result)
//                    }
//                }
//            }
//        )
    }

    // region 核心 Hook 逻辑
    private fun hookTelephonyApis(classLoader: ClassLoader) {
        val tmClass = "android.telephony.TelephonyManager"

        // Hook IMEI/MEID 相关方法
        arrayOf("getDeviceId", "getImei", "getMeid").forEach { method ->
            XposedHelpers.findAndHookMethod(
                tmClass,
                classLoader,
                method,
                object : XC_MethodHook() {
                    override fun afterHookedMethod(param: MethodHookParam) {
                        logCall(
                            caller = tmClass,
                            method = method,
                            result = param.result,
                            stackDepth = STACK_DEPTH
                        )
                    }
                }
            )
        }
    }

    private fun hookSystemApis(classLoader: ClassLoader) {
        // Hook Android ID
        XposedHelpers.findAndHookMethod(
            "android.provider.Settings.Secure",
            classLoader,
            "getString",
            ContentResolver::class.java,
            String::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    if (param.args[1] == "android_id") {
                        logCall(
                            caller = "Settings.Secure",
                            method = "getString(android_id)",
                            result = param.result,
                            stackDepth = STACK_DEPTH
                        )
                    }
                }
            }
        )

        // Hook Build.SERIAL
        XposedHelpers.findAndHookMethod(
            "android.os.Build",
            classLoader,
            "getSerial",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        caller = "Build",
                        method = "getSerial",
                        result = param.result,
                        stackDepth = STACK_DEPTH
                    )
                }
            }
        )
    }

    private fun hookReflectiveCalls(classLoader: ClassLoader) {
        // Hook 反射调用 Build.SERIAL
        XposedHelpers.findAndHookMethod(
            "android.os.Build",
            classLoader,
            "getString",
            String::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    if (param.args[0] == "SERIAL") {
                        logCall(
                            caller = "Build.getString",
                            method = "SERIAL",
                            result = param.result,
                            stackDepth = STACK_DEPTH
                        )
                    }
                }
            }
        )

        // Hook SystemProperties.get()
        XposedHelpers.findAndHookMethod(
            "android.os.SystemProperties",
            classLoader,
            "get",
            String::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    if (param.args[0] == "ro.serialno") {
                        logCall(
                            caller = "SystemProperties",
                            method = "get(ro.serialno)",
                            result = param.result,
                            stackDepth = STACK_DEPTH
                        )
                    }
                }
            }
        )
    }

    private fun bypassPermissionChecks(classLoader: ClassLoader) {
        // 绕过 READ_PHONE_STATE 权限检查
        XposedHelpers.findAndHookMethod(
            "android.app.ContextImpl",
            classLoader,
            "checkPermission",
            String::class.java,
            Int::class.java,
            Int::class.java,
            object : XC_MethodHook() {
                override fun beforeHookedMethod(param: MethodHookParam) {
                    if (param.args[0] == android.Manifest.permission.READ_PHONE_STATE) {
                        param.result = PackageManager.PERMISSION_GRANTED
                    }
                }
            }
        )
    }
    // endregion

    // region 日志工具
    private fun logCall(caller: String, method: String, result: Any?, stackDepth: Int = 0) {
        val stackTrace = if (stackDepth > 0) {
            "\nStackTrace:\n${
                Thread.currentThread().stackTrace
                    .drop(4) // 跳过 Xposed 内部调用
                    .take(stackDepth)
                    .joinToString("\n") { "  at ${it.className}.${it.methodName} (${it.fileName}:${it.lineNumber})" }
            }"
        } else ""

        val logMsg = """
        [][$LOG_TAG] $caller.$method
        → Result: $result
        $stackTrace
        """.trimIndent()

        XposedBridge.log(logMsg)
    }
    // endregion
}