package io.sixminutes.ridicule.monitor

import android.graphics.Point
import android.util.DisplayMetrics
import de.robv.android.xposed.IXposedHookLoadPackage
import de.robv.android.xposed.XC_MethodHook
import de.robv.android.xposed.XposedBridge
import de.robv.android.xposed.XposedHelpers
import de.robv.android.xposed.callbacks.XC_LoadPackage

class ResolutionTracker : IXposedHookLoadPackage {

    companion object {
        private const val TARGET_PACKAGE = "com.target.app"
        private const val LOG_TAG = "ResolutionTracker"
        private const val STACK_DEPTH = 8 // 堆栈跟踪深度
    }

    override fun handleLoadPackage(lpparam: XC_LoadPackage.LoadPackageParam) {
        if (lpparam.packageName != TARGET_PACKAGE) return

        hookDisplayMetrics(lpparam.classLoader)
        hookWindowManagerApis(lpparam.classLoader)
        hookConfigurationApis(lpparam.classLoader)
        hookViewApis(lpparam.classLoader)
    }

    // region 核心 Hook 逻辑
    private fun hookDisplayMetrics(classLoader: ClassLoader) {
        // Hook Resources.getDisplayMetrics()
        XposedHelpers.findAndHookMethod(
            "android.content.res.Resources",
            classLoader,
            "getDisplayMetrics",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    val metrics = param.result as DisplayMetrics
                    logCall(
                        caller = "Resources",
                        method = "getDisplayMetrics",
                        result = "${metrics.widthPixels}x${metrics.heightPixels}",
                        stackDepth = STACK_DEPTH
                    )
                }
            }
        )
    }

    private fun hookWindowManagerApis(classLoader: ClassLoader) {
        // Hook Display.getSize()
        XposedHelpers.findAndHookMethod(
            "android.view.Display",
            classLoader,
            "getSize",
            Point::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    val point = param.args[0] as Point
                    logCall(
                        caller = "Display",
                        method = "getSize",
                        result = "${point.x}x${point.y}",
                        stackDepth = STACK_DEPTH
                    )
                }
            }
        )

        // Hook WindowManager.getDefaultDisplay()
        XposedHelpers.findAndHookMethod(
            "android.view.WindowManager",
            classLoader,
            "getDefaultDisplay",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        caller = "WindowManager",
                        method = "getDefaultDisplay",
                        result = param.result?.javaClass?.name,
                        stackDepth = STACK_DEPTH
                    )
                }
            }
        )
    }

    private fun hookConfigurationApis(classLoader: ClassLoader) {
        // Hook Configuration.screenWidthDp
        XposedHelpers.findAndHookMethod(
            "android.content.res.Configuration",
            classLoader,
            "screenWidthDp",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        caller = "Configuration",
                        method = "screenWidthDp",
                        result = param.result,
                        stackDepth = STACK_DEPTH
                    )
                }
            }
        )

        // Hook screenHeightDp 同理
    }

    private fun hookViewApis(classLoader: ClassLoader) {
        // Hook View 尺寸方法
        arrayOf("getWidth", "getHeight").forEach { method ->
            XposedHelpers.findAndHookMethod(
                "android.view.View",
                classLoader,
                method,
                object : XC_MethodHook() {
                    override fun afterHookedMethod(param: MethodHookParam) {
                        logCall(
                            caller = "View",
                            method = method,
                            result = param.result,
                            stackDepth = STACK_DEPTH
                        )
                    }
                }
            )
        }
    }
    // endregion

    // region 日志工具
    private fun logCall(caller: String, method: String, result: Any?, stackDepth: Int = 0) {
        val stackTrace = if (stackDepth > 0) {
            Thread.currentThread().stackTrace
                .drop(4) // 跳过 Xposed 内部调用
                .take(stackDepth)
                .joinToString("\n") { "  at ${it.className}.${it.methodName} (${it.fileName}:${it.lineNumber})" }
        } else ""

        val logMsg = """
        [][$LOG_TAG] $caller.$method
        → Result: $result
        $stackTrace
        """.trimIndent()

        XposedBridge.log(logMsg)
    }
    // endregion
}